/**
 * @license
 * Copyright 2023 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
import '../../../elevation/elevation.js';
import { LitElement } from 'lit';
/**
 * A card component.
 */
export declare class Card extends LitElement {
    protected render(): import("lit-html").TemplateResult<1>;
}
