// Copyright 2025 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Package soognutil provides interface to Android soong.
// See
//
//	https://android.googlesource.com/platform/build/soong/
package soongutil
