// Copyright 2026 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v7.35.0
// source: deps_log_record.proto

package proto

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type PathRecord struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Id            int64                  `protobuf:"varint,1,opt,name=id,proto3" json:"id,omitempty"`
	Pathname      string                 `protobuf:"bytes,2,opt,name=pathname,proto3" json:"pathname,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PathRecord) Reset() {
	*x = PathRecord{}
	mi := &file_deps_log_record_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PathRecord) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PathRecord) ProtoMessage() {}

func (x *PathRecord) ProtoReflect() protoreflect.Message {
	mi := &file_deps_log_record_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PathRecord.ProtoReflect.Descriptor instead.
func (*PathRecord) Descriptor() ([]byte, []int) {
	return file_deps_log_record_proto_rawDescGZIP(), []int{0}
}

func (x *PathRecord) GetId() int64 {
	if x != nil {
		return x.Id
	}
	return 0
}

func (x *PathRecord) GetPathname() string {
	if x != nil {
		return x.Pathname
	}
	return ""
}

type DepsRecord struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	OutId         int64                  `protobuf:"varint,1,opt,name=out_id,json=outId,proto3" json:"out_id,omitempty"`
	OutMtime      int64                  `protobuf:"varint,2,opt,name=out_mtime,json=outMtime,proto3" json:"out_mtime,omitempty"` // unix nano sec.
	OutHash       string                 `protobuf:"bytes,3,opt,name=out_hash,json=outHash,proto3" json:"out_hash,omitempty"`
	OutSizeBytes  int64                  `protobuf:"varint,4,opt,name=out_size_bytes,json=outSizeBytes,proto3" json:"out_size_bytes,omitempty"`
	InputIds      []int64                `protobuf:"varint,5,rep,packed,name=input_ids,json=inputIds,proto3" json:"input_ids,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DepsRecord) Reset() {
	*x = DepsRecord{}
	mi := &file_deps_log_record_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DepsRecord) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DepsRecord) ProtoMessage() {}

func (x *DepsRecord) ProtoReflect() protoreflect.Message {
	mi := &file_deps_log_record_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DepsRecord.ProtoReflect.Descriptor instead.
func (*DepsRecord) Descriptor() ([]byte, []int) {
	return file_deps_log_record_proto_rawDescGZIP(), []int{1}
}

func (x *DepsRecord) GetOutId() int64 {
	if x != nil {
		return x.OutId
	}
	return 0
}

func (x *DepsRecord) GetOutMtime() int64 {
	if x != nil {
		return x.OutMtime
	}
	return 0
}

func (x *DepsRecord) GetOutHash() string {
	if x != nil {
		return x.OutHash
	}
	return ""
}

func (x *DepsRecord) GetOutSizeBytes() int64 {
	if x != nil {
		return x.OutSizeBytes
	}
	return 0
}

func (x *DepsRecord) GetInputIds() []int64 {
	if x != nil {
		return x.InputIds
	}
	return nil
}

type DepsLogRecord struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// pathnames.
	// assign path id from beginning of file.
	Paths []*PathRecord `protobuf:"bytes,1,rep,name=paths,proto3" json:"paths,omitempty"`
	// deps record. i.e. output -> {inputs}
	Deps          []*DepsRecord `protobuf:"bytes,2,rep,name=deps,proto3" json:"deps,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DepsLogRecord) Reset() {
	*x = DepsLogRecord{}
	mi := &file_deps_log_record_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DepsLogRecord) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DepsLogRecord) ProtoMessage() {}

func (x *DepsLogRecord) ProtoReflect() protoreflect.Message {
	mi := &file_deps_log_record_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DepsLogRecord.ProtoReflect.Descriptor instead.
func (*DepsLogRecord) Descriptor() ([]byte, []int) {
	return file_deps_log_record_proto_rawDescGZIP(), []int{2}
}

func (x *DepsLogRecord) GetPaths() []*PathRecord {
	if x != nil {
		return x.Paths
	}
	return nil
}

func (x *DepsLogRecord) GetDeps() []*DepsRecord {
	if x != nil {
		return x.Deps
	}
	return nil
}

var File_deps_log_record_proto protoreflect.FileDescriptor

const file_deps_log_record_proto_rawDesc = "" +
	"\n" +
	"\x15deps_log_record.proto\x12\x15siso.build.ninjabuild\"8\n" +
	"\n" +
	"PathRecord\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\x03R\x02id\x12\x1a\n" +
	"\bpathname\x18\x02 \x01(\tR\bpathname\"\x9e\x01\n" +
	"\n" +
	"DepsRecord\x12\x15\n" +
	"\x06out_id\x18\x01 \x01(\x03R\x05outId\x12\x1b\n" +
	"\tout_mtime\x18\x02 \x01(\x03R\boutMtime\x12\x19\n" +
	"\bout_hash\x18\x03 \x01(\tR\aoutHash\x12$\n" +
	"\x0eout_size_bytes\x18\x04 \x01(\x03R\foutSizeBytes\x12\x1b\n" +
	"\tinput_ids\x18\x05 \x03(\x03R\binputIds\"\x7f\n" +
	"\rDepsLogRecord\x127\n" +
	"\x05paths\x18\x01 \x03(\v2!.siso.build.ninjabuild.PathRecordR\x05paths\x125\n" +
	"\x04deps\x18\x02 \x03(\v2!.siso.build.ninjabuild.DepsRecordR\x04depsB3Z1go.chromium.org/build/siso/build/ninjabuild/protob\x06proto3"

var (
	file_deps_log_record_proto_rawDescOnce sync.Once
	file_deps_log_record_proto_rawDescData []byte
)

func file_deps_log_record_proto_rawDescGZIP() []byte {
	file_deps_log_record_proto_rawDescOnce.Do(func() {
		file_deps_log_record_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_deps_log_record_proto_rawDesc), len(file_deps_log_record_proto_rawDesc)))
	})
	return file_deps_log_record_proto_rawDescData
}

var file_deps_log_record_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_deps_log_record_proto_goTypes = []any{
	(*PathRecord)(nil),    // 0: siso.build.ninjabuild.PathRecord
	(*DepsRecord)(nil),    // 1: siso.build.ninjabuild.DepsRecord
	(*DepsLogRecord)(nil), // 2: siso.build.ninjabuild.DepsLogRecord
}
var file_deps_log_record_proto_depIdxs = []int32{
	0, // 0: siso.build.ninjabuild.DepsLogRecord.paths:type_name -> siso.build.ninjabuild.PathRecord
	1, // 1: siso.build.ninjabuild.DepsLogRecord.deps:type_name -> siso.build.ninjabuild.DepsRecord
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_deps_log_record_proto_init() }
func file_deps_log_record_proto_init() {
	if File_deps_log_record_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_deps_log_record_proto_rawDesc), len(file_deps_log_record_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_deps_log_record_proto_goTypes,
		DependencyIndexes: file_deps_log_record_proto_depIdxs,
		MessageInfos:      file_deps_log_record_proto_msgTypes,
	}.Build()
	File_deps_log_record_proto = out.File
	file_deps_log_record_proto_goTypes = nil
	file_deps_log_record_proto_depIdxs = nil
}
