/**
 * @license
 * Copyright 2021 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
import '../../field/outlined-field.js';
import { TextField } from './text-field.js';
/**
 * An outlined text field component
 */
export declare class OutlinedTextField extends TextField {
    protected readonly fieldTag: import("lit-html/static.js").StaticValue;
}
