/**
 * @license
 * Copyright 2021 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
import { __decorate } from '../../../../tslib/tslib.js';
import { customElement } from '../../../../lit/decorators.js';
import { NavigationTab } from './internal/navigation-tab.js';
import { styles } from './internal/navigation-tab-styles.js';
/**
 * @final
 * @suppress {visibility}
 */
export let MdNavigationTab = class MdNavigationTab extends NavigationTab {
};
MdNavigationTab.styles = [styles];
MdNavigationTab = __decorate([
    customElement('md-navigation-tab')
], MdNavigationTab);
//# sourceMappingURL=navigation-tab.js.map