// Copyright 2026 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Package lockfile provides a file-based locking mechanism, allowing
// synchronisation across multiple processes.
package lockfile
