/* Copyright 2024 The Chromium Authors
 * Use of this source code is governed by a BSD-style license that can be
 * found in the LICENSE file. */

.dark-high-contrast {
  --md-sys-color-primary: rgb(246 255 218);
  --md-sys-color-surface-tint: rgb(183 208 133);
  --md-sys-color-on-primary: rgb(0 0 0);
  --md-sys-color-primary-container: rgb(187 212 137);
  --md-sys-color-on-primary-container: rgb(0 0 0);
  --md-sys-color-secondary: rgb(246 255 221);
  --md-sys-color-on-secondary: rgb(0 0 0);
  --md-sys-color-secondary-container: rgb(198 206 175);
  --md-sys-color-on-secondary-container: rgb(0 0 0);
  --md-sys-color-tertiary: rgb(235 255 251);
  --md-sys-color-on-tertiary: rgb(0 0 0);
  --md-sys-color-tertiary-container: rgb(165 212 204);
  --md-sys-color-on-tertiary-container: rgb(0 0 0);
  --md-sys-color-error: rgb(255 249 249);
  --md-sys-color-on-error: rgb(0 0 0);
  --md-sys-color-error-container: rgb(255 186 177);
  --md-sys-color-on-error-container: rgb(0 0 0);
  --md-sys-color-background: rgb(18 20 13);
  --md-sys-color-on-background: rgb(227 227 216);
  --md-sys-color-surface: rgb(18 20 13);
  --md-sys-color-on-surface: rgb(255 255 255);
  --md-sys-color-surface-variant: rgb(69 72 60);
  --md-sys-color-on-surface-variant: rgb(250 252 236);
  --md-sys-color-outline: rgb(202 204 189);
  --md-sys-color-outline-variant: rgb(202 204 189);
  --md-sys-color-shadow: rgb(0 0 0);
  --md-sys-color-scrim: rgb(0 0 0);
  --md-sys-color-inverse-surface: rgb(227 227 216);
  --md-sys-color-inverse-on-surface: rgb(0 0 0);
  --md-sys-color-inverse-primary: rgb(32 47 0);
  --md-sys-color-primary-fixed: rgb(215 240 162);
  --md-sys-color-on-primary-fixed: rgb(0 0 0);
  --md-sys-color-primary-fixed-dim: rgb(187 212 137);
  --md-sys-color-on-primary-fixed-variant: rgb(16 25 0);
  --md-sys-color-secondary-fixed: rgb(226 235 202);
  --md-sys-color-on-secondary-fixed: rgb(0 0 0);
  --md-sys-color-secondary-fixed-dim: rgb(198 206 175);
  --md-sys-color-on-secondary-fixed-variant: rgb(18 25 5);
  --md-sys-color-tertiary-fixed: rgb(192 240 233);
  --md-sys-color-on-tertiary-fixed: rgb(0 0 0);
  --md-sys-color-tertiary-fixed-dim: rgb(165 212 204);
  --md-sys-color-on-tertiary-fixed-variant: rgb(0 26 23);
  --md-sys-color-surface-dim: rgb(18 20 13);
  --md-sys-color-surface-bright: rgb(56 58 50);
  --md-sys-color-surface-container-lowest: rgb(13 15 9);
  --md-sys-color-surface-container-low: rgb(26 28 21);
  --md-sys-color-surface-container: rgb(30 32 25);
  --md-sys-color-surface-container-high: rgb(41 43 35);
  --md-sys-color-surface-container-highest: rgb(52 54 46);
}
