/**
 * @license
 * Copyright 2022 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
export declare const stringConverter: {
    fromAttribute(value: string | null): string;
    toAttribute(value: string): string | null;
};
