{{/* Copyright 2024 The Chromium Authors
Use of this source code is governed by a BSD-style license that can be
found in the LICENSE file. */}}

{{define "content"}}
{{/* These IDs are used in JavaScript to extract errors from our own responses. */}}
<h2 id="page-error-title">{{.errorTitle}}</h2>
<div id="page-error-message">{{.errorMessage}}</div>
{{end}}
