// Copyright 2017 Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// copy from https://googleplex-android.googlesource.com/platform/build/soong/+/afa116d431cea6fc77cea243f1b024daf8596a64/ui/status/ninja_frontend/frontend.proto

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v7.35.0
// source: frontend.proto

package proto

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Status_Message_Level int32

const (
	Status_Message_INFO    Status_Message_Level = 0
	Status_Message_WARNING Status_Message_Level = 1
	Status_Message_ERROR   Status_Message_Level = 2
	Status_Message_DEBUG   Status_Message_Level = 3
)

// Enum value maps for Status_Message_Level.
var (
	Status_Message_Level_name = map[int32]string{
		0: "INFO",
		1: "WARNING",
		2: "ERROR",
		3: "DEBUG",
	}
	Status_Message_Level_value = map[string]int32{
		"INFO":    0,
		"WARNING": 1,
		"ERROR":   2,
		"DEBUG":   3,
	}
)

func (x Status_Message_Level) Enum() *Status_Message_Level {
	p := new(Status_Message_Level)
	*p = x
	return p
}

func (x Status_Message_Level) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (Status_Message_Level) Descriptor() protoreflect.EnumDescriptor {
	return file_frontend_proto_enumTypes[0].Descriptor()
}

func (Status_Message_Level) Type() protoreflect.EnumType {
	return &file_frontend_proto_enumTypes[0]
}

func (x Status_Message_Level) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Do not use.
func (x *Status_Message_Level) UnmarshalJSON(b []byte) error {
	num, err := protoimpl.X.UnmarshalJSONEnum(x.Descriptor(), b)
	if err != nil {
		return err
	}
	*x = Status_Message_Level(num)
	return nil
}

// Deprecated: Use Status_Message_Level.Descriptor instead.
func (Status_Message_Level) EnumDescriptor() ([]byte, []int) {
	return file_frontend_proto_rawDescGZIP(), []int{0, 5, 0}
}

type Status struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	TotalEdges    *Status_TotalEdges     `protobuf:"bytes,1,opt,name=total_edges,json=totalEdges" json:"total_edges,omitempty"`
	BuildStarted  *Status_BuildStarted   `protobuf:"bytes,2,opt,name=build_started,json=buildStarted" json:"build_started,omitempty"`
	BuildFinished *Status_BuildFinished  `protobuf:"bytes,3,opt,name=build_finished,json=buildFinished" json:"build_finished,omitempty"`
	EdgeStarted   *Status_EdgeStarted    `protobuf:"bytes,4,opt,name=edge_started,json=edgeStarted" json:"edge_started,omitempty"`
	EdgeFinished  *Status_EdgeFinished   `protobuf:"bytes,5,opt,name=edge_finished,json=edgeFinished" json:"edge_finished,omitempty"`
	Message       *Status_Message        `protobuf:"bytes,6,opt,name=message" json:"message,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Status) Reset() {
	*x = Status{}
	mi := &file_frontend_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Status) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Status) ProtoMessage() {}

func (x *Status) ProtoReflect() protoreflect.Message {
	mi := &file_frontend_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Status.ProtoReflect.Descriptor instead.
func (*Status) Descriptor() ([]byte, []int) {
	return file_frontend_proto_rawDescGZIP(), []int{0}
}

func (x *Status) GetTotalEdges() *Status_TotalEdges {
	if x != nil {
		return x.TotalEdges
	}
	return nil
}

func (x *Status) GetBuildStarted() *Status_BuildStarted {
	if x != nil {
		return x.BuildStarted
	}
	return nil
}

func (x *Status) GetBuildFinished() *Status_BuildFinished {
	if x != nil {
		return x.BuildFinished
	}
	return nil
}

func (x *Status) GetEdgeStarted() *Status_EdgeStarted {
	if x != nil {
		return x.EdgeStarted
	}
	return nil
}

func (x *Status) GetEdgeFinished() *Status_EdgeFinished {
	if x != nil {
		return x.EdgeFinished
	}
	return nil
}

func (x *Status) GetMessage() *Status_Message {
	if x != nil {
		return x.Message
	}
	return nil
}

type Status_TotalEdges struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// New value for total edges in the build.
	TotalEdges    *uint32 `protobuf:"varint,1,opt,name=total_edges,json=totalEdges" json:"total_edges,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Status_TotalEdges) Reset() {
	*x = Status_TotalEdges{}
	mi := &file_frontend_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Status_TotalEdges) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Status_TotalEdges) ProtoMessage() {}

func (x *Status_TotalEdges) ProtoReflect() protoreflect.Message {
	mi := &file_frontend_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Status_TotalEdges.ProtoReflect.Descriptor instead.
func (*Status_TotalEdges) Descriptor() ([]byte, []int) {
	return file_frontend_proto_rawDescGZIP(), []int{0, 0}
}

func (x *Status_TotalEdges) GetTotalEdges() uint32 {
	if x != nil && x.TotalEdges != nil {
		return *x.TotalEdges
	}
	return 0
}

type Status_BuildStarted struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Number of jobs Ninja will run in parallel.
	Parallelism *uint32 `protobuf:"varint,1,opt,name=parallelism" json:"parallelism,omitempty"`
	// Verbose value passed to ninja.
	Verbose *bool `protobuf:"varint,2,opt,name=verbose" json:"verbose,omitempty"`
	// Critical path's running time in milliseconds
	CriticalPathTime *uint32 `protobuf:"varint,3,opt,name=critical_path_time,json=criticalPathTime" json:"critical_path_time,omitempty"`
	// Total running time of every need-to-build edge in milliseconds
	EstimatedTotalTime *uint32 `protobuf:"varint,4,opt,name=estimated_total_time,json=estimatedTotalTime" json:"estimated_total_time,omitempty"`
	unknownFields      protoimpl.UnknownFields
	sizeCache          protoimpl.SizeCache
}

func (x *Status_BuildStarted) Reset() {
	*x = Status_BuildStarted{}
	mi := &file_frontend_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Status_BuildStarted) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Status_BuildStarted) ProtoMessage() {}

func (x *Status_BuildStarted) ProtoReflect() protoreflect.Message {
	mi := &file_frontend_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Status_BuildStarted.ProtoReflect.Descriptor instead.
func (*Status_BuildStarted) Descriptor() ([]byte, []int) {
	return file_frontend_proto_rawDescGZIP(), []int{0, 1}
}

func (x *Status_BuildStarted) GetParallelism() uint32 {
	if x != nil && x.Parallelism != nil {
		return *x.Parallelism
	}
	return 0
}

func (x *Status_BuildStarted) GetVerbose() bool {
	if x != nil && x.Verbose != nil {
		return *x.Verbose
	}
	return false
}

func (x *Status_BuildStarted) GetCriticalPathTime() uint32 {
	if x != nil && x.CriticalPathTime != nil {
		return *x.CriticalPathTime
	}
	return 0
}

func (x *Status_BuildStarted) GetEstimatedTotalTime() uint32 {
	if x != nil && x.EstimatedTotalTime != nil {
		return *x.EstimatedTotalTime
	}
	return 0
}

type Status_BuildFinished struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Status_BuildFinished) Reset() {
	*x = Status_BuildFinished{}
	mi := &file_frontend_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Status_BuildFinished) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Status_BuildFinished) ProtoMessage() {}

func (x *Status_BuildFinished) ProtoReflect() protoreflect.Message {
	mi := &file_frontend_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Status_BuildFinished.ProtoReflect.Descriptor instead.
func (*Status_BuildFinished) Descriptor() ([]byte, []int) {
	return file_frontend_proto_rawDescGZIP(), []int{0, 2}
}

type Status_EdgeStarted struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Edge identification number, unique to a Ninja run.
	Id *uint32 `protobuf:"varint,1,opt,name=id" json:"id,omitempty"`
	// Edge start time in milliseconds since Ninja started.
	StartTime *uint32 `protobuf:"varint,2,opt,name=start_time,json=startTime" json:"start_time,omitempty"`
	// List of edge inputs.
	Inputs []string `protobuf:"bytes,3,rep,name=inputs" json:"inputs,omitempty"`
	// List of edge outputs.
	Outputs []string `protobuf:"bytes,4,rep,name=outputs" json:"outputs,omitempty"`
	// Description field from the edge.
	Desc *string `protobuf:"bytes,5,opt,name=desc" json:"desc,omitempty"`
	// Command field from the edge.
	Command *string `protobuf:"bytes,6,opt,name=command" json:"command,omitempty"`
	// Edge uses console.
	Console *bool `protobuf:"varint,7,opt,name=console" json:"console,omitempty"`
	// Changed inputs since last build.
	ChangedInputs []string `protobuf:"bytes,8,rep,name=changed_inputs,json=changedInputs" json:"changed_inputs,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Status_EdgeStarted) Reset() {
	*x = Status_EdgeStarted{}
	mi := &file_frontend_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Status_EdgeStarted) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Status_EdgeStarted) ProtoMessage() {}

func (x *Status_EdgeStarted) ProtoReflect() protoreflect.Message {
	mi := &file_frontend_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Status_EdgeStarted.ProtoReflect.Descriptor instead.
func (*Status_EdgeStarted) Descriptor() ([]byte, []int) {
	return file_frontend_proto_rawDescGZIP(), []int{0, 3}
}

func (x *Status_EdgeStarted) GetId() uint32 {
	if x != nil && x.Id != nil {
		return *x.Id
	}
	return 0
}

func (x *Status_EdgeStarted) GetStartTime() uint32 {
	if x != nil && x.StartTime != nil {
		return *x.StartTime
	}
	return 0
}

func (x *Status_EdgeStarted) GetInputs() []string {
	if x != nil {
		return x.Inputs
	}
	return nil
}

func (x *Status_EdgeStarted) GetOutputs() []string {
	if x != nil {
		return x.Outputs
	}
	return nil
}

func (x *Status_EdgeStarted) GetDesc() string {
	if x != nil && x.Desc != nil {
		return *x.Desc
	}
	return ""
}

func (x *Status_EdgeStarted) GetCommand() string {
	if x != nil && x.Command != nil {
		return *x.Command
	}
	return ""
}

func (x *Status_EdgeStarted) GetConsole() bool {
	if x != nil && x.Console != nil {
		return *x.Console
	}
	return false
}

func (x *Status_EdgeStarted) GetChangedInputs() []string {
	if x != nil {
		return x.ChangedInputs
	}
	return nil
}

type Status_EdgeFinished struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Edge identification number, unique to a Ninja run.
	Id *uint32 `protobuf:"varint,1,opt,name=id" json:"id,omitempty"`
	// Edge end time in milliseconds since Ninja started.
	EndTime *uint32 `protobuf:"varint,2,opt,name=end_time,json=endTime" json:"end_time,omitempty"`
	// Exit status (0 for success).
	Status *int32 `protobuf:"zigzag32,3,opt,name=status" json:"status,omitempty"`
	// Edge output, may contain ANSI codes.
	Output *string `protobuf:"bytes,4,opt,name=output" json:"output,omitempty"`
	// Number of milliseconds spent executing in user mode
	UserTime *uint32 `protobuf:"varint,5,opt,name=user_time,json=userTime" json:"user_time,omitempty"`
	// Number of milliseconds spent executing in kernel mode
	SystemTime *uint32 `protobuf:"varint,6,opt,name=system_time,json=systemTime" json:"system_time,omitempty"`
	// Max resident set size in kB
	MaxRssKb *uint64 `protobuf:"varint,7,opt,name=max_rss_kb,json=maxRssKb" json:"max_rss_kb,omitempty"`
	// Minor page faults
	MinorPageFaults *uint64 `protobuf:"varint,8,opt,name=minor_page_faults,json=minorPageFaults" json:"minor_page_faults,omitempty"`
	// Major page faults
	MajorPageFaults *uint64 `protobuf:"varint,9,opt,name=major_page_faults,json=majorPageFaults" json:"major_page_faults,omitempty"`
	// IO input in kB
	IoInputKb *uint64 `protobuf:"varint,10,opt,name=io_input_kb,json=ioInputKb" json:"io_input_kb,omitempty"`
	// IO output in kB
	IoOutputKb *uint64 `protobuf:"varint,11,opt,name=io_output_kb,json=ioOutputKb" json:"io_output_kb,omitempty"`
	// Voluntary context switches
	VoluntaryContextSwitches *uint64 `protobuf:"varint,12,opt,name=voluntary_context_switches,json=voluntaryContextSwitches" json:"voluntary_context_switches,omitempty"`
	// Involuntary context switches
	InvoluntaryContextSwitches *uint64 `protobuf:"varint,13,opt,name=involuntary_context_switches,json=involuntaryContextSwitches" json:"involuntary_context_switches,omitempty"`
	// Arbitrary tags for build system profiling (module names and types, rule
	// names, etc). Format of the string is implementation defined.
	Tags *string `protobuf:"bytes,14,opt,name=tags" json:"tags,omitempty"`
	// True if edge was canceled.
	Canceled      *bool `protobuf:"varint,15,opt,name=canceled" json:"canceled,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Status_EdgeFinished) Reset() {
	*x = Status_EdgeFinished{}
	mi := &file_frontend_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Status_EdgeFinished) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Status_EdgeFinished) ProtoMessage() {}

func (x *Status_EdgeFinished) ProtoReflect() protoreflect.Message {
	mi := &file_frontend_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Status_EdgeFinished.ProtoReflect.Descriptor instead.
func (*Status_EdgeFinished) Descriptor() ([]byte, []int) {
	return file_frontend_proto_rawDescGZIP(), []int{0, 4}
}

func (x *Status_EdgeFinished) GetId() uint32 {
	if x != nil && x.Id != nil {
		return *x.Id
	}
	return 0
}

func (x *Status_EdgeFinished) GetEndTime() uint32 {
	if x != nil && x.EndTime != nil {
		return *x.EndTime
	}
	return 0
}

func (x *Status_EdgeFinished) GetStatus() int32 {
	if x != nil && x.Status != nil {
		return *x.Status
	}
	return 0
}

func (x *Status_EdgeFinished) GetOutput() string {
	if x != nil && x.Output != nil {
		return *x.Output
	}
	return ""
}

func (x *Status_EdgeFinished) GetUserTime() uint32 {
	if x != nil && x.UserTime != nil {
		return *x.UserTime
	}
	return 0
}

func (x *Status_EdgeFinished) GetSystemTime() uint32 {
	if x != nil && x.SystemTime != nil {
		return *x.SystemTime
	}
	return 0
}

func (x *Status_EdgeFinished) GetMaxRssKb() uint64 {
	if x != nil && x.MaxRssKb != nil {
		return *x.MaxRssKb
	}
	return 0
}

func (x *Status_EdgeFinished) GetMinorPageFaults() uint64 {
	if x != nil && x.MinorPageFaults != nil {
		return *x.MinorPageFaults
	}
	return 0
}

func (x *Status_EdgeFinished) GetMajorPageFaults() uint64 {
	if x != nil && x.MajorPageFaults != nil {
		return *x.MajorPageFaults
	}
	return 0
}

func (x *Status_EdgeFinished) GetIoInputKb() uint64 {
	if x != nil && x.IoInputKb != nil {
		return *x.IoInputKb
	}
	return 0
}

func (x *Status_EdgeFinished) GetIoOutputKb() uint64 {
	if x != nil && x.IoOutputKb != nil {
		return *x.IoOutputKb
	}
	return 0
}

func (x *Status_EdgeFinished) GetVoluntaryContextSwitches() uint64 {
	if x != nil && x.VoluntaryContextSwitches != nil {
		return *x.VoluntaryContextSwitches
	}
	return 0
}

func (x *Status_EdgeFinished) GetInvoluntaryContextSwitches() uint64 {
	if x != nil && x.InvoluntaryContextSwitches != nil {
		return *x.InvoluntaryContextSwitches
	}
	return 0
}

func (x *Status_EdgeFinished) GetTags() string {
	if x != nil && x.Tags != nil {
		return *x.Tags
	}
	return ""
}

func (x *Status_EdgeFinished) GetCanceled() bool {
	if x != nil && x.Canceled != nil {
		return *x.Canceled
	}
	return false
}

type Status_Message struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Message priority level (DEBUG, INFO, WARNING, ERROR).
	Level *Status_Message_Level `protobuf:"varint,1,opt,name=level,enum=ninja.Status_Message_Level,def=0" json:"level,omitempty"`
	// Info/warning/error message from Ninja.
	Message       *string `protobuf:"bytes,2,opt,name=message" json:"message,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

// Default values for Status_Message fields.
const (
	Default_Status_Message_Level = Status_Message_INFO
)

func (x *Status_Message) Reset() {
	*x = Status_Message{}
	mi := &file_frontend_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Status_Message) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Status_Message) ProtoMessage() {}

func (x *Status_Message) ProtoReflect() protoreflect.Message {
	mi := &file_frontend_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Status_Message.ProtoReflect.Descriptor instead.
func (*Status_Message) Descriptor() ([]byte, []int) {
	return file_frontend_proto_rawDescGZIP(), []int{0, 5}
}

func (x *Status_Message) GetLevel() Status_Message_Level {
	if x != nil && x.Level != nil {
		return *x.Level
	}
	return Default_Status_Message_Level
}

func (x *Status_Message) GetMessage() string {
	if x != nil && x.Message != nil {
		return *x.Message
	}
	return ""
}

var File_frontend_proto protoreflect.FileDescriptor

const file_frontend_proto_rawDesc = "" +
	"\n" +
	"\x0efrontend.proto\x12\x05ninja\"\xec\v\n" +
	"\x06Status\x129\n" +
	"\vtotal_edges\x18\x01 \x01(\v2\x18.ninja.Status.TotalEdgesR\n" +
	"totalEdges\x12?\n" +
	"\rbuild_started\x18\x02 \x01(\v2\x1a.ninja.Status.BuildStartedR\fbuildStarted\x12B\n" +
	"\x0ebuild_finished\x18\x03 \x01(\v2\x1b.ninja.Status.BuildFinishedR\rbuildFinished\x12<\n" +
	"\fedge_started\x18\x04 \x01(\v2\x19.ninja.Status.EdgeStartedR\vedgeStarted\x12?\n" +
	"\redge_finished\x18\x05 \x01(\v2\x1a.ninja.Status.EdgeFinishedR\fedgeFinished\x12/\n" +
	"\amessage\x18\x06 \x01(\v2\x15.ninja.Status.MessageR\amessage\x1a-\n" +
	"\n" +
	"TotalEdges\x12\x1f\n" +
	"\vtotal_edges\x18\x01 \x01(\rR\n" +
	"totalEdges\x1a\xaa\x01\n" +
	"\fBuildStarted\x12 \n" +
	"\vparallelism\x18\x01 \x01(\rR\vparallelism\x12\x18\n" +
	"\averbose\x18\x02 \x01(\bR\averbose\x12,\n" +
	"\x12critical_path_time\x18\x03 \x01(\rR\x10criticalPathTime\x120\n" +
	"\x14estimated_total_time\x18\x04 \x01(\rR\x12estimatedTotalTime\x1a\x0f\n" +
	"\rBuildFinished\x1a\xdd\x01\n" +
	"\vEdgeStarted\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\rR\x02id\x12\x1d\n" +
	"\n" +
	"start_time\x18\x02 \x01(\rR\tstartTime\x12\x16\n" +
	"\x06inputs\x18\x03 \x03(\tR\x06inputs\x12\x18\n" +
	"\aoutputs\x18\x04 \x03(\tR\aoutputs\x12\x12\n" +
	"\x04desc\x18\x05 \x01(\tR\x04desc\x12\x18\n" +
	"\acommand\x18\x06 \x01(\tR\acommand\x12\x18\n" +
	"\aconsole\x18\a \x01(\bR\aconsole\x12%\n" +
	"\x0echanged_inputs\x18\b \x03(\tR\rchangedInputs\x1a\x8f\x04\n" +
	"\fEdgeFinished\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\rR\x02id\x12\x19\n" +
	"\bend_time\x18\x02 \x01(\rR\aendTime\x12\x16\n" +
	"\x06status\x18\x03 \x01(\x11R\x06status\x12\x16\n" +
	"\x06output\x18\x04 \x01(\tR\x06output\x12\x1b\n" +
	"\tuser_time\x18\x05 \x01(\rR\buserTime\x12\x1f\n" +
	"\vsystem_time\x18\x06 \x01(\rR\n" +
	"systemTime\x12\x1c\n" +
	"\n" +
	"max_rss_kb\x18\a \x01(\x04R\bmaxRssKb\x12*\n" +
	"\x11minor_page_faults\x18\b \x01(\x04R\x0fminorPageFaults\x12*\n" +
	"\x11major_page_faults\x18\t \x01(\x04R\x0fmajorPageFaults\x12\x1e\n" +
	"\vio_input_kb\x18\n" +
	" \x01(\x04R\tioInputKb\x12 \n" +
	"\fio_output_kb\x18\v \x01(\x04R\n" +
	"ioOutputKb\x12<\n" +
	"\x1avoluntary_context_switches\x18\f \x01(\x04R\x18voluntaryContextSwitches\x12@\n" +
	"\x1cinvoluntary_context_switches\x18\r \x01(\x04R\x1ainvoluntaryContextSwitches\x12\x12\n" +
	"\x04tags\x18\x0e \x01(\tR\x04tags\x12\x1a\n" +
	"\bcanceled\x18\x0f \x01(\bR\bcanceled\x1a\x92\x01\n" +
	"\aMessage\x127\n" +
	"\x05level\x18\x01 \x01(\x0e2\x1b.ninja.Status.Message.Level:\x04INFOR\x05level\x12\x18\n" +
	"\amessage\x18\x02 \x01(\tR\amessage\"4\n" +
	"\x05Level\x12\b\n" +
	"\x04INFO\x10\x00\x12\v\n" +
	"\aWARNING\x10\x01\x12\t\n" +
	"\x05ERROR\x10\x02\x12\t\n" +
	"\x05DEBUG\x10\x03B:H\x03Z6go.chromium.org/build/siso/toolsupport/soongutil/proto"

var (
	file_frontend_proto_rawDescOnce sync.Once
	file_frontend_proto_rawDescData []byte
)

func file_frontend_proto_rawDescGZIP() []byte {
	file_frontend_proto_rawDescOnce.Do(func() {
		file_frontend_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_frontend_proto_rawDesc), len(file_frontend_proto_rawDesc)))
	})
	return file_frontend_proto_rawDescData
}

var file_frontend_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_frontend_proto_msgTypes = make([]protoimpl.MessageInfo, 7)
var file_frontend_proto_goTypes = []any{
	(Status_Message_Level)(0),    // 0: ninja.Status.Message.Level
	(*Status)(nil),               // 1: ninja.Status
	(*Status_TotalEdges)(nil),    // 2: ninja.Status.TotalEdges
	(*Status_BuildStarted)(nil),  // 3: ninja.Status.BuildStarted
	(*Status_BuildFinished)(nil), // 4: ninja.Status.BuildFinished
	(*Status_EdgeStarted)(nil),   // 5: ninja.Status.EdgeStarted
	(*Status_EdgeFinished)(nil),  // 6: ninja.Status.EdgeFinished
	(*Status_Message)(nil),       // 7: ninja.Status.Message
}
var file_frontend_proto_depIdxs = []int32{
	2, // 0: ninja.Status.total_edges:type_name -> ninja.Status.TotalEdges
	3, // 1: ninja.Status.build_started:type_name -> ninja.Status.BuildStarted
	4, // 2: ninja.Status.build_finished:type_name -> ninja.Status.BuildFinished
	5, // 3: ninja.Status.edge_started:type_name -> ninja.Status.EdgeStarted
	6, // 4: ninja.Status.edge_finished:type_name -> ninja.Status.EdgeFinished
	7, // 5: ninja.Status.message:type_name -> ninja.Status.Message
	0, // 6: ninja.Status.Message.level:type_name -> ninja.Status.Message.Level
	7, // [7:7] is the sub-list for method output_type
	7, // [7:7] is the sub-list for method input_type
	7, // [7:7] is the sub-list for extension type_name
	7, // [7:7] is the sub-list for extension extendee
	0, // [0:7] is the sub-list for field type_name
}

func init() { file_frontend_proto_init() }
func file_frontend_proto_init() {
	if File_frontend_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_frontend_proto_rawDesc), len(file_frontend_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   7,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_frontend_proto_goTypes,
		DependencyIndexes: file_frontend_proto_depIdxs,
		EnumInfos:         file_frontend_proto_enumTypes,
		MessageInfos:      file_frontend_proto_msgTypes,
	}.Build()
	File_frontend_proto = out.File
	file_frontend_proto_goTypes = nil
	file_frontend_proto_depIdxs = nil
}
