# Copyright 2024 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.


def main():
  # flatc generates gen/generated.h
  with open("gen/generated.h", "w") as w:
    w.write('generated by flatc')
