/**
 * @license
 * Copyright 2021 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
import '../../elevation/elevation.js';
import { Button } from './button.js';
/**
 * An elevated button component.
 */
export declare class ElevatedButton extends Button {
    protected renderElevationOrOutline(): import("lit-html").TemplateResult<1>;
}
