// Copyright 2026 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Package cartfsutil provides access to cartfs.
// See
//
//	go/cartfs
//	sso://tbi-internal/artfs - source code
package cartfsutil
