// Copyright 2026 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Package nsjailutil provides utiliti4es of nsjail.
//
// It will set up similar nsjail as Andoid's ninja.
// b/448484009 e.g. http://ag/36083741
package nsjailutil
