/**
 * @license
 * Copyright 2022 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
import { LitElement } from 'lit';
/**
 * b/265340196 - add docs
 */
export declare class Badge extends LitElement {
    value: string;
    protected render(): import("lit-html").TemplateResult<1>;
}
