# Copyright 2023 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import sys
with open(sys.argv[1], "w") as f:
  f.write("")
