// Copyright 2026 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Package proto provides protocol buffer message for cartfs.
// cartfs proto can be found at sso://tbi-internal/artfs proto/ directory.
package proto
