// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v7.35.0
// source: proto/server/cartfs.proto

package server

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type CartfsState int32

const (
	CartfsState_STATE_UNSPECIFIED   CartfsState = 0
	CartfsState_STATE_STARTING_UP   CartfsState = 1
	CartfsState_STATE_RUNNING       CartfsState = 2
	CartfsState_STATE_SHUTTING_DOWN CartfsState = 3
	CartfsState_STATE_ERROR         CartfsState = 4
)

// Enum value maps for CartfsState.
var (
	CartfsState_name = map[int32]string{
		0: "STATE_UNSPECIFIED",
		1: "STATE_STARTING_UP",
		2: "STATE_RUNNING",
		3: "STATE_SHUTTING_DOWN",
		4: "STATE_ERROR",
	}
	CartfsState_value = map[string]int32{
		"STATE_UNSPECIFIED":   0,
		"STATE_STARTING_UP":   1,
		"STATE_RUNNING":       2,
		"STATE_SHUTTING_DOWN": 3,
		"STATE_ERROR":         4,
	}
)

func (x CartfsState) Enum() *CartfsState {
	p := new(CartfsState)
	*p = x
	return p
}

func (x CartfsState) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (CartfsState) Descriptor() protoreflect.EnumDescriptor {
	return file_proto_server_cartfs_proto_enumTypes[0].Descriptor()
}

func (CartfsState) Type() protoreflect.EnumType {
	return &file_proto_server_cartfs_proto_enumTypes[0]
}

func (x CartfsState) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use CartfsState.Descriptor instead.
func (CartfsState) EnumDescriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{0}
}

// The urgency with which content should be pulled from the remote CAS.
// IMMEDIATE: The content should be pulled as soon as possible.
// QUEUED: The content should be pulled when there is capacity.
// ON_ACCESS: The content should be pulled when it is accessed.
type ContentPullUrgency int32

const (
	ContentPullUrgency_CONTENT_PULL_URGENCY_UNSPECIFIED ContentPullUrgency = 0
	ContentPullUrgency_CONTENT_PULL_URGENCY_IMMEDIATE   ContentPullUrgency = 1
	ContentPullUrgency_CONTENT_PULL_URGENCY_QUEUED      ContentPullUrgency = 2
	ContentPullUrgency_CONTENT_PULL_URGENCY_ON_ACCESS   ContentPullUrgency = 3
)

// Enum value maps for ContentPullUrgency.
var (
	ContentPullUrgency_name = map[int32]string{
		0: "CONTENT_PULL_URGENCY_UNSPECIFIED",
		1: "CONTENT_PULL_URGENCY_IMMEDIATE",
		2: "CONTENT_PULL_URGENCY_QUEUED",
		3: "CONTENT_PULL_URGENCY_ON_ACCESS",
	}
	ContentPullUrgency_value = map[string]int32{
		"CONTENT_PULL_URGENCY_UNSPECIFIED": 0,
		"CONTENT_PULL_URGENCY_IMMEDIATE":   1,
		"CONTENT_PULL_URGENCY_QUEUED":      2,
		"CONTENT_PULL_URGENCY_ON_ACCESS":   3,
	}
)

func (x ContentPullUrgency) Enum() *ContentPullUrgency {
	p := new(ContentPullUrgency)
	*p = x
	return p
}

func (x ContentPullUrgency) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ContentPullUrgency) Descriptor() protoreflect.EnumDescriptor {
	return file_proto_server_cartfs_proto_enumTypes[1].Descriptor()
}

func (ContentPullUrgency) Type() protoreflect.EnumType {
	return &file_proto_server_cartfs_proto_enumTypes[1]
}

func (x ContentPullUrgency) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ContentPullUrgency.Descriptor instead.
func (ContentPullUrgency) EnumDescriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{1}
}

type CopyDirectoryResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Success       bool                   `protobuf:"varint,1,opt,name=success,proto3" json:"success,omitempty"`
	ErrorMessage  string                 `protobuf:"bytes,2,opt,name=error_message,json=errorMessage,proto3" json:"error_message,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CopyDirectoryResponse) Reset() {
	*x = CopyDirectoryResponse{}
	mi := &file_proto_server_cartfs_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CopyDirectoryResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CopyDirectoryResponse) ProtoMessage() {}

func (x *CopyDirectoryResponse) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CopyDirectoryResponse.ProtoReflect.Descriptor instead.
func (*CopyDirectoryResponse) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{0}
}

func (x *CopyDirectoryResponse) GetSuccess() bool {
	if x != nil {
		return x.Success
	}
	return false
}

func (x *CopyDirectoryResponse) GetErrorMessage() string {
	if x != nil {
		return x.ErrorMessage
	}
	return ""
}

type CopyDirectoryRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	FromPath      string                 `protobuf:"bytes,1,opt,name=from_path,json=fromPath,proto3" json:"from_path,omitempty"`
	ToPath        string                 `protobuf:"bytes,2,opt,name=to_path,json=toPath,proto3" json:"to_path,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CopyDirectoryRequest) Reset() {
	*x = CopyDirectoryRequest{}
	mi := &file_proto_server_cartfs_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CopyDirectoryRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CopyDirectoryRequest) ProtoMessage() {}

func (x *CopyDirectoryRequest) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CopyDirectoryRequest.ProtoReflect.Descriptor instead.
func (*CopyDirectoryRequest) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{1}
}

func (x *CopyDirectoryRequest) GetFromPath() string {
	if x != nil {
		return x.FromPath
	}
	return ""
}

func (x *CopyDirectoryRequest) GetToPath() string {
	if x != nil {
		return x.ToPath
	}
	return ""
}

type AuditLogRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Inode         uint64                 `protobuf:"varint,1,opt,name=inode,proto3" json:"inode,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AuditLogRequest) Reset() {
	*x = AuditLogRequest{}
	mi := &file_proto_server_cartfs_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AuditLogRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AuditLogRequest) ProtoMessage() {}

func (x *AuditLogRequest) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AuditLogRequest.ProtoReflect.Descriptor instead.
func (*AuditLogRequest) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{2}
}

func (x *AuditLogRequest) GetInode() uint64 {
	if x != nil {
		return x.Inode
	}
	return 0
}

type AuditLogResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Message       string                 `protobuf:"bytes,1,opt,name=message,proto3" json:"message,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AuditLogResponse) Reset() {
	*x = AuditLogResponse{}
	mi := &file_proto_server_cartfs_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AuditLogResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AuditLogResponse) ProtoMessage() {}

func (x *AuditLogResponse) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AuditLogResponse.ProtoReflect.Descriptor instead.
func (*AuditLogResponse) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{3}
}

func (x *AuditLogResponse) GetMessage() string {
	if x != nil {
		return x.Message
	}
	return ""
}

type GetStateRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetStateRequest) Reset() {
	*x = GetStateRequest{}
	mi := &file_proto_server_cartfs_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetStateRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetStateRequest) ProtoMessage() {}

func (x *GetStateRequest) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetStateRequest.ProtoReflect.Descriptor instead.
func (*GetStateRequest) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{4}
}

type GetStateResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	State         CartfsState            `protobuf:"varint,1,opt,name=state,proto3,enum=cartfs.CartfsState" json:"state,omitempty"`
	MountPoint    string                 `protobuf:"bytes,2,opt,name=mount_point,json=mountPoint,proto3" json:"mount_point,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetStateResponse) Reset() {
	*x = GetStateResponse{}
	mi := &file_proto_server_cartfs_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetStateResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetStateResponse) ProtoMessage() {}

func (x *GetStateResponse) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetStateResponse.ProtoReflect.Descriptor instead.
func (*GetStateResponse) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{5}
}

func (x *GetStateResponse) GetState() CartfsState {
	if x != nil {
		return x.State
	}
	return CartfsState_STATE_UNSPECIFIED
}

func (x *GetStateResponse) GetMountPoint() string {
	if x != nil {
		return x.MountPoint
	}
	return ""
}

type LocalCASStatusRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *LocalCASStatusRequest) Reset() {
	*x = LocalCASStatusRequest{}
	mi := &file_proto_server_cartfs_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LocalCASStatusRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LocalCASStatusRequest) ProtoMessage() {}

func (x *LocalCASStatusRequest) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LocalCASStatusRequest.ProtoReflect.Descriptor instead.
func (*LocalCASStatusRequest) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{6}
}

type LocalCASStatusResponse struct {
	state                 protoimpl.MessageState `protogen:"open.v1"`
	TotalOrphanedFiles    uint64                 `protobuf:"varint,1,opt,name=total_orphaned_files,json=totalOrphanedFiles,proto3" json:"total_orphaned_files,omitempty"`
	TotalOrphanedFileSize uint64                 `protobuf:"varint,2,opt,name=total_orphaned_file_size,json=totalOrphanedFileSize,proto3" json:"total_orphaned_file_size,omitempty"`
	unknownFields         protoimpl.UnknownFields
	sizeCache             protoimpl.SizeCache
}

func (x *LocalCASStatusResponse) Reset() {
	*x = LocalCASStatusResponse{}
	mi := &file_proto_server_cartfs_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LocalCASStatusResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LocalCASStatusResponse) ProtoMessage() {}

func (x *LocalCASStatusResponse) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LocalCASStatusResponse.ProtoReflect.Descriptor instead.
func (*LocalCASStatusResponse) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{7}
}

func (x *LocalCASStatusResponse) GetTotalOrphanedFiles() uint64 {
	if x != nil {
		return x.TotalOrphanedFiles
	}
	return 0
}

func (x *LocalCASStatusResponse) GetTotalOrphanedFileSize() uint64 {
	if x != nil {
		return x.TotalOrphanedFileSize
	}
	return 0
}

// Requests the digest of a file, either by the inode or the path.  If possible,
// the inode is preferred as it is more efficient.
type GetDigestRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Identifier:
	//
	//	*GetDigestRequest_Inode
	//	*GetDigestRequest_Path
	Identifier    isGetDigestRequest_Identifier `protobuf_oneof:"identifier"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetDigestRequest) Reset() {
	*x = GetDigestRequest{}
	mi := &file_proto_server_cartfs_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetDigestRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetDigestRequest) ProtoMessage() {}

func (x *GetDigestRequest) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetDigestRequest.ProtoReflect.Descriptor instead.
func (*GetDigestRequest) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{8}
}

func (x *GetDigestRequest) GetIdentifier() isGetDigestRequest_Identifier {
	if x != nil {
		return x.Identifier
	}
	return nil
}

func (x *GetDigestRequest) GetInode() uint64 {
	if x != nil {
		if x, ok := x.Identifier.(*GetDigestRequest_Inode); ok {
			return x.Inode
		}
	}
	return 0
}

func (x *GetDigestRequest) GetPath() string {
	if x != nil {
		if x, ok := x.Identifier.(*GetDigestRequest_Path); ok {
			return x.Path
		}
	}
	return ""
}

type isGetDigestRequest_Identifier interface {
	isGetDigestRequest_Identifier()
}

type GetDigestRequest_Inode struct {
	Inode uint64 `protobuf:"varint,1,opt,name=inode,proto3,oneof"`
}

type GetDigestRequest_Path struct {
	Path string `protobuf:"bytes,2,opt,name=path,proto3,oneof"`
}

func (*GetDigestRequest_Inode) isGetDigestRequest_Identifier() {}

func (*GetDigestRequest_Path) isGetDigestRequest_Identifier() {}

type GetDigestResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Hash          string                 `protobuf:"bytes,1,opt,name=hash,proto3" json:"hash,omitempty"`
	Size          uint64                 `protobuf:"varint,2,opt,name=size,proto3" json:"size,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetDigestResponse) Reset() {
	*x = GetDigestResponse{}
	mi := &file_proto_server_cartfs_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetDigestResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetDigestResponse) ProtoMessage() {}

func (x *GetDigestResponse) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetDigestResponse.ProtoReflect.Descriptor instead.
func (*GetDigestResponse) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{9}
}

func (x *GetDigestResponse) GetHash() string {
	if x != nil {
		return x.Hash
	}
	return ""
}

func (x *GetDigestResponse) GetSize() uint64 {
	if x != nil {
		return x.Size
	}
	return 0
}

type FileRegistrationInfo struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Path of the file within the CartFs mount point.
	// Both relative (e.g., "a/b/c") and leading slash paths (e.g., "/a/b/c")
	// are supported and strictly resolved relative to the VFS mount root.
	// Parent directory traversals (..) are rejected for safety.
	Path string `protobuf:"bytes,1,opt,name=path,proto3" json:"path,omitempty"`
	// The hash of the file.  Currently this is a sha256 hash.
	Hash string `protobuf:"bytes,2,opt,name=hash,proto3" json:"hash,omitempty"`
	// The size of the file in bytes.
	Size uint64 `protobuf:"varint,3,opt,name=size,proto3" json:"size,omitempty"`
	// The urgency with which the content should be pulled from the remote CAS.
	Urgency ContentPullUrgency `protobuf:"varint,4,opt,name=urgency,proto3,enum=cartfs.ContentPullUrgency" json:"urgency,omitempty"`
	// Whether the file is executable.
	IsExecutable  bool `protobuf:"varint,5,opt,name=is_executable,json=isExecutable,proto3" json:"is_executable,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FileRegistrationInfo) Reset() {
	*x = FileRegistrationInfo{}
	mi := &file_proto_server_cartfs_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FileRegistrationInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FileRegistrationInfo) ProtoMessage() {}

func (x *FileRegistrationInfo) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FileRegistrationInfo.ProtoReflect.Descriptor instead.
func (*FileRegistrationInfo) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{10}
}

func (x *FileRegistrationInfo) GetPath() string {
	if x != nil {
		return x.Path
	}
	return ""
}

func (x *FileRegistrationInfo) GetHash() string {
	if x != nil {
		return x.Hash
	}
	return ""
}

func (x *FileRegistrationInfo) GetSize() uint64 {
	if x != nil {
		return x.Size
	}
	return 0
}

func (x *FileRegistrationInfo) GetUrgency() ContentPullUrgency {
	if x != nil {
		return x.Urgency
	}
	return ContentPullUrgency_CONTENT_PULL_URGENCY_UNSPECIFIED
}

func (x *FileRegistrationInfo) GetIsExecutable() bool {
	if x != nil {
		return x.IsExecutable
	}
	return false
}

type RegisterFilesRequest struct {
	state         protoimpl.MessageState  `protogen:"open.v1"`
	Registrations []*FileRegistrationInfo `protobuf:"bytes,1,rep,name=registrations,proto3" json:"registrations,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RegisterFilesRequest) Reset() {
	*x = RegisterFilesRequest{}
	mi := &file_proto_server_cartfs_proto_msgTypes[11]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RegisterFilesRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RegisterFilesRequest) ProtoMessage() {}

func (x *RegisterFilesRequest) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[11]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RegisterFilesRequest.ProtoReflect.Descriptor instead.
func (*RegisterFilesRequest) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{11}
}

func (x *RegisterFilesRequest) GetRegistrations() []*FileRegistrationInfo {
	if x != nil {
		return x.Registrations
	}
	return nil
}

type RegistrationError struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Path          string                 `protobuf:"bytes,1,opt,name=path,proto3" json:"path,omitempty"`
	ErrorMessage  string                 `protobuf:"bytes,2,opt,name=error_message,json=errorMessage,proto3" json:"error_message,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RegistrationError) Reset() {
	*x = RegistrationError{}
	mi := &file_proto_server_cartfs_proto_msgTypes[12]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RegistrationError) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RegistrationError) ProtoMessage() {}

func (x *RegistrationError) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[12]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RegistrationError.ProtoReflect.Descriptor instead.
func (*RegistrationError) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{12}
}

func (x *RegistrationError) GetPath() string {
	if x != nil {
		return x.Path
	}
	return ""
}

func (x *RegistrationError) GetErrorMessage() string {
	if x != nil {
		return x.ErrorMessage
	}
	return ""
}

type RegisterFilesResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Errors        []*RegistrationError   `protobuf:"bytes,1,rep,name=errors,proto3" json:"errors,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RegisterFilesResponse) Reset() {
	*x = RegisterFilesResponse{}
	mi := &file_proto_server_cartfs_proto_msgTypes[13]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RegisterFilesResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RegisterFilesResponse) ProtoMessage() {}

func (x *RegisterFilesResponse) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[13]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RegisterFilesResponse.ProtoReflect.Descriptor instead.
func (*RegisterFilesResponse) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{13}
}

func (x *RegisterFilesResponse) GetErrors() []*RegistrationError {
	if x != nil {
		return x.Errors
	}
	return nil
}

type CreateWorkspaceRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// GoB repo name, e.g. googleplex-android/platform/manifest.
	// TODO - myriamanis: Ensure it's working for repo url as well as GoB repo name.
	RepoId          string `protobuf:"bytes,1,opt,name=repo_id,json=repoId,proto3" json:"repo_id,omitempty"`
	RefOrCommitHash string `protobuf:"bytes,2,opt,name=ref_or_commit_hash,json=refOrCommitHash,proto3" json:"ref_or_commit_hash,omitempty"`
	// e.g. projects/git-workspaces-rbe-614681/instances/default_instance.
	RbeCasInstance string `protobuf:"bytes,3,opt,name=rbe_cas_instance,json=rbeCasInstance,proto3" json:"rbe_cas_instance,omitempty"`
	// The name of the workspace.
	WorkspaceName string `protobuf:"bytes,4,opt,name=workspace_name,json=workspaceName,proto3" json:"workspace_name,omitempty"`
	// If set, will attempt to materialise from repo manifest file in the commit identified
	// by `repo_id` and `ref_or_commit_hash`.
	ManifestFilePath string `protobuf:"bytes,5,opt,name=manifest_file_path,json=manifestFilePath,proto3" json:"manifest_file_path,omitempty"`
	// Workspace context fields
	Uid           uint32 `protobuf:"varint,6,opt,name=uid,proto3" json:"uid,omitempty"`
	Gid           uint32 `protobuf:"varint,7,opt,name=gid,proto3" json:"gid,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateWorkspaceRequest) Reset() {
	*x = CreateWorkspaceRequest{}
	mi := &file_proto_server_cartfs_proto_msgTypes[14]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateWorkspaceRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateWorkspaceRequest) ProtoMessage() {}

func (x *CreateWorkspaceRequest) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[14]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateWorkspaceRequest.ProtoReflect.Descriptor instead.
func (*CreateWorkspaceRequest) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{14}
}

func (x *CreateWorkspaceRequest) GetRepoId() string {
	if x != nil {
		return x.RepoId
	}
	return ""
}

func (x *CreateWorkspaceRequest) GetRefOrCommitHash() string {
	if x != nil {
		return x.RefOrCommitHash
	}
	return ""
}

func (x *CreateWorkspaceRequest) GetRbeCasInstance() string {
	if x != nil {
		return x.RbeCasInstance
	}
	return ""
}

func (x *CreateWorkspaceRequest) GetWorkspaceName() string {
	if x != nil {
		return x.WorkspaceName
	}
	return ""
}

func (x *CreateWorkspaceRequest) GetManifestFilePath() string {
	if x != nil {
		return x.ManifestFilePath
	}
	return ""
}

func (x *CreateWorkspaceRequest) GetUid() uint32 {
	if x != nil {
		return x.Uid
	}
	return 0
}

func (x *CreateWorkspaceRequest) GetGid() uint32 {
	if x != nil {
		return x.Gid
	}
	return 0
}

type CreateWorkspaceResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateWorkspaceResponse) Reset() {
	*x = CreateWorkspaceResponse{}
	mi := &file_proto_server_cartfs_proto_msgTypes[15]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateWorkspaceResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateWorkspaceResponse) ProtoMessage() {}

func (x *CreateWorkspaceResponse) ProtoReflect() protoreflect.Message {
	mi := &file_proto_server_cartfs_proto_msgTypes[15]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateWorkspaceResponse.ProtoReflect.Descriptor instead.
func (*CreateWorkspaceResponse) Descriptor() ([]byte, []int) {
	return file_proto_server_cartfs_proto_rawDescGZIP(), []int{15}
}

var File_proto_server_cartfs_proto protoreflect.FileDescriptor

const file_proto_server_cartfs_proto_rawDesc = "" +
	"\n" +
	"\x19proto/server/cartfs.proto\x12\x06cartfs\x1a\x16proto/server/tap.proto\"V\n" +
	"\x15CopyDirectoryResponse\x12\x18\n" +
	"\asuccess\x18\x01 \x01(\bR\asuccess\x12#\n" +
	"\rerror_message\x18\x02 \x01(\tR\ferrorMessage\"L\n" +
	"\x14CopyDirectoryRequest\x12\x1b\n" +
	"\tfrom_path\x18\x01 \x01(\tR\bfromPath\x12\x17\n" +
	"\ato_path\x18\x02 \x01(\tR\x06toPath\"'\n" +
	"\x0fAuditLogRequest\x12\x14\n" +
	"\x05inode\x18\x01 \x01(\x04R\x05inode\",\n" +
	"\x10AuditLogResponse\x12\x18\n" +
	"\amessage\x18\x01 \x01(\tR\amessage\"\x11\n" +
	"\x0fGetStateRequest\"^\n" +
	"\x10GetStateResponse\x12)\n" +
	"\x05state\x18\x01 \x01(\x0e2\x13.cartfs.CartfsStateR\x05state\x12\x1f\n" +
	"\vmount_point\x18\x02 \x01(\tR\n" +
	"mountPoint\"\x17\n" +
	"\x15LocalCASStatusRequest\"\x83\x01\n" +
	"\x16LocalCASStatusResponse\x120\n" +
	"\x14total_orphaned_files\x18\x01 \x01(\x04R\x12totalOrphanedFiles\x127\n" +
	"\x18total_orphaned_file_size\x18\x02 \x01(\x04R\x15totalOrphanedFileSize\"N\n" +
	"\x10GetDigestRequest\x12\x16\n" +
	"\x05inode\x18\x01 \x01(\x04H\x00R\x05inode\x12\x14\n" +
	"\x04path\x18\x02 \x01(\tH\x00R\x04pathB\f\n" +
	"\n" +
	"identifier\";\n" +
	"\x11GetDigestResponse\x12\x12\n" +
	"\x04hash\x18\x01 \x01(\tR\x04hash\x12\x12\n" +
	"\x04size\x18\x02 \x01(\x04R\x04size\"\xad\x01\n" +
	"\x14FileRegistrationInfo\x12\x12\n" +
	"\x04path\x18\x01 \x01(\tR\x04path\x12\x12\n" +
	"\x04hash\x18\x02 \x01(\tR\x04hash\x12\x12\n" +
	"\x04size\x18\x03 \x01(\x04R\x04size\x124\n" +
	"\aurgency\x18\x04 \x01(\x0e2\x1a.cartfs.ContentPullUrgencyR\aurgency\x12#\n" +
	"\ris_executable\x18\x05 \x01(\bR\fisExecutable\"Z\n" +
	"\x14RegisterFilesRequest\x12B\n" +
	"\rregistrations\x18\x01 \x03(\v2\x1c.cartfs.FileRegistrationInfoR\rregistrations\"L\n" +
	"\x11RegistrationError\x12\x12\n" +
	"\x04path\x18\x01 \x01(\tR\x04path\x12#\n" +
	"\rerror_message\x18\x02 \x01(\tR\ferrorMessage\"J\n" +
	"\x15RegisterFilesResponse\x121\n" +
	"\x06errors\x18\x01 \x03(\v2\x19.cartfs.RegistrationErrorR\x06errors\"\x81\x02\n" +
	"\x16CreateWorkspaceRequest\x12\x17\n" +
	"\arepo_id\x18\x01 \x01(\tR\x06repoId\x12+\n" +
	"\x12ref_or_commit_hash\x18\x02 \x01(\tR\x0frefOrCommitHash\x12(\n" +
	"\x10rbe_cas_instance\x18\x03 \x01(\tR\x0erbeCasInstance\x12%\n" +
	"\x0eworkspace_name\x18\x04 \x01(\tR\rworkspaceName\x12,\n" +
	"\x12manifest_file_path\x18\x05 \x01(\tR\x10manifestFilePath\x12\x10\n" +
	"\x03uid\x18\x06 \x01(\rR\x03uid\x12\x10\n" +
	"\x03gid\x18\a \x01(\rR\x03gid\"\x19\n" +
	"\x17CreateWorkspaceResponse*x\n" +
	"\vCartfsState\x12\x15\n" +
	"\x11STATE_UNSPECIFIED\x10\x00\x12\x15\n" +
	"\x11STATE_STARTING_UP\x10\x01\x12\x11\n" +
	"\rSTATE_RUNNING\x10\x02\x12\x17\n" +
	"\x13STATE_SHUTTING_DOWN\x10\x03\x12\x0f\n" +
	"\vSTATE_ERROR\x10\x04*\xa3\x01\n" +
	"\x12ContentPullUrgency\x12$\n" +
	" CONTENT_PULL_URGENCY_UNSPECIFIED\x10\x00\x12\"\n" +
	"\x1eCONTENT_PULL_URGENCY_IMMEDIATE\x10\x01\x12\x1f\n" +
	"\x1bCONTENT_PULL_URGENCY_QUEUED\x10\x02\x12\"\n" +
	"\x1eCONTENT_PULL_URGENCY_ON_ACCESS\x10\x032\xb8\x06\n" +
	"\x06Cartfs\x12N\n" +
	"\rCopyDirectory\x12\x1c.cartfs.CopyDirectoryRequest\x1a\x1d.cartfs.CopyDirectoryResponse\"\x00\x12?\n" +
	"\bAuditLog\x12\x17.cartfs.AuditLogRequest\x1a\x18.cartfs.AuditLogResponse\"\x00\x12?\n" +
	"\bGetState\x12\x17.cartfs.GetStateRequest\x1a\x18.cartfs.GetStateResponse\"\x00\x12Q\n" +
	"\x0eLocalCASStatus\x12\x1d.cartfs.LocalCASStatusRequest\x1a\x1e.cartfs.LocalCASStatusResponse\"\x00\x12B\n" +
	"\tGetDigest\x12\x18.cartfs.GetDigestRequest\x1a\x19.cartfs.GetDigestResponse\"\x00\x12N\n" +
	"\rRegisterFiles\x12\x1c.cartfs.RegisterFilesRequest\x1a\x1d.cartfs.RegisterFilesResponse\"\x00\x12L\n" +
	"\x0fStartTapSession\x12\x1e.cartfs.StartTapSessionRequest\x1a\x15.cartfs.TapEventBatch\"\x000\x01\x12W\n" +
	"\x10FinishTapSession\x12\x1f.cartfs.FinishTapSessionRequest\x1a .cartfs.FinishTapSessionResponse\"\x00\x12x\n" +
	"\x1bExcludeProcessGroupFromTaps\x12*.cartfs.ExcludeProcessGroupFromTapsRequest\x1a+.cartfs.ExcludeProcessGroupFromTapsResponse\"\x00\x12T\n" +
	"\x0fCreateWorkspace\x12\x1e.cartfs.CreateWorkspaceRequest\x1a\x1f.cartfs.CreateWorkspaceResponse\"\x00B@Z>go.chromium.org/build/siso/toolsupport/cartfsutil/proto/serverb\x06proto3"

var (
	file_proto_server_cartfs_proto_rawDescOnce sync.Once
	file_proto_server_cartfs_proto_rawDescData []byte
)

func file_proto_server_cartfs_proto_rawDescGZIP() []byte {
	file_proto_server_cartfs_proto_rawDescOnce.Do(func() {
		file_proto_server_cartfs_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_proto_server_cartfs_proto_rawDesc), len(file_proto_server_cartfs_proto_rawDesc)))
	})
	return file_proto_server_cartfs_proto_rawDescData
}

var file_proto_server_cartfs_proto_enumTypes = make([]protoimpl.EnumInfo, 2)
var file_proto_server_cartfs_proto_msgTypes = make([]protoimpl.MessageInfo, 16)
var file_proto_server_cartfs_proto_goTypes = []any{
	(CartfsState)(0),                            // 0: cartfs.CartfsState
	(ContentPullUrgency)(0),                     // 1: cartfs.ContentPullUrgency
	(*CopyDirectoryResponse)(nil),               // 2: cartfs.CopyDirectoryResponse
	(*CopyDirectoryRequest)(nil),                // 3: cartfs.CopyDirectoryRequest
	(*AuditLogRequest)(nil),                     // 4: cartfs.AuditLogRequest
	(*AuditLogResponse)(nil),                    // 5: cartfs.AuditLogResponse
	(*GetStateRequest)(nil),                     // 6: cartfs.GetStateRequest
	(*GetStateResponse)(nil),                    // 7: cartfs.GetStateResponse
	(*LocalCASStatusRequest)(nil),               // 8: cartfs.LocalCASStatusRequest
	(*LocalCASStatusResponse)(nil),              // 9: cartfs.LocalCASStatusResponse
	(*GetDigestRequest)(nil),                    // 10: cartfs.GetDigestRequest
	(*GetDigestResponse)(nil),                   // 11: cartfs.GetDigestResponse
	(*FileRegistrationInfo)(nil),                // 12: cartfs.FileRegistrationInfo
	(*RegisterFilesRequest)(nil),                // 13: cartfs.RegisterFilesRequest
	(*RegistrationError)(nil),                   // 14: cartfs.RegistrationError
	(*RegisterFilesResponse)(nil),               // 15: cartfs.RegisterFilesResponse
	(*CreateWorkspaceRequest)(nil),              // 16: cartfs.CreateWorkspaceRequest
	(*CreateWorkspaceResponse)(nil),             // 17: cartfs.CreateWorkspaceResponse
	(*StartTapSessionRequest)(nil),              // 18: cartfs.StartTapSessionRequest
	(*FinishTapSessionRequest)(nil),             // 19: cartfs.FinishTapSessionRequest
	(*ExcludeProcessGroupFromTapsRequest)(nil),  // 20: cartfs.ExcludeProcessGroupFromTapsRequest
	(*TapEventBatch)(nil),                       // 21: cartfs.TapEventBatch
	(*FinishTapSessionResponse)(nil),            // 22: cartfs.FinishTapSessionResponse
	(*ExcludeProcessGroupFromTapsResponse)(nil), // 23: cartfs.ExcludeProcessGroupFromTapsResponse
}
var file_proto_server_cartfs_proto_depIdxs = []int32{
	0,  // 0: cartfs.GetStateResponse.state:type_name -> cartfs.CartfsState
	1,  // 1: cartfs.FileRegistrationInfo.urgency:type_name -> cartfs.ContentPullUrgency
	12, // 2: cartfs.RegisterFilesRequest.registrations:type_name -> cartfs.FileRegistrationInfo
	14, // 3: cartfs.RegisterFilesResponse.errors:type_name -> cartfs.RegistrationError
	3,  // 4: cartfs.Cartfs.CopyDirectory:input_type -> cartfs.CopyDirectoryRequest
	4,  // 5: cartfs.Cartfs.AuditLog:input_type -> cartfs.AuditLogRequest
	6,  // 6: cartfs.Cartfs.GetState:input_type -> cartfs.GetStateRequest
	8,  // 7: cartfs.Cartfs.LocalCASStatus:input_type -> cartfs.LocalCASStatusRequest
	10, // 8: cartfs.Cartfs.GetDigest:input_type -> cartfs.GetDigestRequest
	13, // 9: cartfs.Cartfs.RegisterFiles:input_type -> cartfs.RegisterFilesRequest
	18, // 10: cartfs.Cartfs.StartTapSession:input_type -> cartfs.StartTapSessionRequest
	19, // 11: cartfs.Cartfs.FinishTapSession:input_type -> cartfs.FinishTapSessionRequest
	20, // 12: cartfs.Cartfs.ExcludeProcessGroupFromTaps:input_type -> cartfs.ExcludeProcessGroupFromTapsRequest
	16, // 13: cartfs.Cartfs.CreateWorkspace:input_type -> cartfs.CreateWorkspaceRequest
	2,  // 14: cartfs.Cartfs.CopyDirectory:output_type -> cartfs.CopyDirectoryResponse
	5,  // 15: cartfs.Cartfs.AuditLog:output_type -> cartfs.AuditLogResponse
	7,  // 16: cartfs.Cartfs.GetState:output_type -> cartfs.GetStateResponse
	9,  // 17: cartfs.Cartfs.LocalCASStatus:output_type -> cartfs.LocalCASStatusResponse
	11, // 18: cartfs.Cartfs.GetDigest:output_type -> cartfs.GetDigestResponse
	15, // 19: cartfs.Cartfs.RegisterFiles:output_type -> cartfs.RegisterFilesResponse
	21, // 20: cartfs.Cartfs.StartTapSession:output_type -> cartfs.TapEventBatch
	22, // 21: cartfs.Cartfs.FinishTapSession:output_type -> cartfs.FinishTapSessionResponse
	23, // 22: cartfs.Cartfs.ExcludeProcessGroupFromTaps:output_type -> cartfs.ExcludeProcessGroupFromTapsResponse
	17, // 23: cartfs.Cartfs.CreateWorkspace:output_type -> cartfs.CreateWorkspaceResponse
	14, // [14:24] is the sub-list for method output_type
	4,  // [4:14] is the sub-list for method input_type
	4,  // [4:4] is the sub-list for extension type_name
	4,  // [4:4] is the sub-list for extension extendee
	0,  // [0:4] is the sub-list for field type_name
}

func init() { file_proto_server_cartfs_proto_init() }
func file_proto_server_cartfs_proto_init() {
	if File_proto_server_cartfs_proto != nil {
		return
	}
	file_proto_server_tap_proto_init()
	file_proto_server_cartfs_proto_msgTypes[8].OneofWrappers = []any{
		(*GetDigestRequest_Inode)(nil),
		(*GetDigestRequest_Path)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_proto_server_cartfs_proto_rawDesc), len(file_proto_server_cartfs_proto_rawDesc)),
			NumEnums:      2,
			NumMessages:   16,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_proto_server_cartfs_proto_goTypes,
		DependencyIndexes: file_proto_server_cartfs_proto_depIdxs,
		EnumInfos:         file_proto_server_cartfs_proto_enumTypes,
		MessageInfos:      file_proto_server_cartfs_proto_msgTypes,
	}.Build()
	File_proto_server_cartfs_proto = out.File
	file_proto_server_cartfs_proto_goTypes = nil
	file_proto_server_cartfs_proto_depIdxs = nil
}
