// Copyright 2025 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Package cipdutil provides interface to cipd.
//
// https://pkg.go.dev/go.chromium.org/luci/cipd/version
package cipdutil
