// Copyright 2025 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Package collector provides an OTEL (Open Telemetry) executor that
// runs OTEL Collector (https://opentelemetry.io/docs/collector/)

// The collector is used to abstract out the metrics and logs upload
// processes outside of Siso execution.
package collector
