// Copyright 2025 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

//go:build !linux

package localexec

import "context"

func oomScoreAdj(ctx context.Context, pid int, score int) {}
