// Copyright 2018 The Bazel Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.33.4
// source: build/bazel/semver/semver.proto

package semver

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The full version of a given tool.
type SemVer struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The major version, e.g 10 for 10.2.3.
	Major int32 `protobuf:"varint,1,opt,name=major,proto3" json:"major,omitempty"`
	// The minor version, e.g. 2 for 10.2.3.
	Minor int32 `protobuf:"varint,2,opt,name=minor,proto3" json:"minor,omitempty"`
	// The patch version, e.g 3 for 10.2.3.
	Patch int32 `protobuf:"varint,3,opt,name=patch,proto3" json:"patch,omitempty"`
	// The pre-release version. Either this field or major/minor/patch fields
	// must be filled. They are mutually exclusive. Pre-release versions are
	// assumed to be earlier than any released versions.
	Prerelease    string `protobuf:"bytes,4,opt,name=prerelease,proto3" json:"prerelease,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SemVer) Reset() {
	*x = SemVer{}
	mi := &file_build_bazel_semver_semver_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SemVer) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SemVer) ProtoMessage() {}

func (x *SemVer) ProtoReflect() protoreflect.Message {
	mi := &file_build_bazel_semver_semver_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SemVer.ProtoReflect.Descriptor instead.
func (*SemVer) Descriptor() ([]byte, []int) {
	return file_build_bazel_semver_semver_proto_rawDescGZIP(), []int{0}
}

func (x *SemVer) GetMajor() int32 {
	if x != nil {
		return x.Major
	}
	return 0
}

func (x *SemVer) GetMinor() int32 {
	if x != nil {
		return x.Minor
	}
	return 0
}

func (x *SemVer) GetPatch() int32 {
	if x != nil {
		return x.Patch
	}
	return 0
}

func (x *SemVer) GetPrerelease() string {
	if x != nil {
		return x.Prerelease
	}
	return ""
}

var File_build_bazel_semver_semver_proto protoreflect.FileDescriptor

const file_build_bazel_semver_semver_proto_rawDesc = "" +
	"\n" +
	"\x1fbuild/bazel/semver/semver.proto\x12\x12build.bazel.semver\"j\n" +
	"\x06SemVer\x12\x14\n" +
	"\x05major\x18\x01 \x01(\x05R\x05major\x12\x14\n" +
	"\x05minor\x18\x02 \x01(\x05R\x05minor\x12\x14\n" +
	"\x05patch\x18\x03 \x01(\x05R\x05patch\x12\x1e\n" +
	"\n" +
	"prerelease\x18\x04 \x01(\tR\n" +
	"prereleaseBt\n" +
	"\x12build.bazel.semverB\vSemverProtoP\x01Z4github.com/bazelbuild/remote-apis/build/bazel/semver\xa2\x02\x03SMV\xaa\x02\x12Build.Bazel.Semverb\x06proto3"

var (
	file_build_bazel_semver_semver_proto_rawDescOnce sync.Once
	file_build_bazel_semver_semver_proto_rawDescData []byte
)

func file_build_bazel_semver_semver_proto_rawDescGZIP() []byte {
	file_build_bazel_semver_semver_proto_rawDescOnce.Do(func() {
		file_build_bazel_semver_semver_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_build_bazel_semver_semver_proto_rawDesc), len(file_build_bazel_semver_semver_proto_rawDesc)))
	})
	return file_build_bazel_semver_semver_proto_rawDescData
}

var file_build_bazel_semver_semver_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_build_bazel_semver_semver_proto_goTypes = []any{
	(*SemVer)(nil), // 0: build.bazel.semver.SemVer
}
var file_build_bazel_semver_semver_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_build_bazel_semver_semver_proto_init() }
func file_build_bazel_semver_semver_proto_init() {
	if File_build_bazel_semver_semver_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_build_bazel_semver_semver_proto_rawDesc), len(file_build_bazel_semver_semver_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_build_bazel_semver_semver_proto_goTypes,
		DependencyIndexes: file_build_bazel_semver_semver_proto_depIdxs,
		MessageInfos:      file_build_bazel_semver_semver_proto_msgTypes,
	}.Build()
	File_build_bazel_semver_semver_proto = out.File
	file_build_bazel_semver_semver_proto_goTypes = nil
	file_build_bazel_semver_semver_proto_depIdxs = nil
}
